@@ -14,25 +14,25 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 #include <ext/spl/spl_iterators.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "phongo_compat.h"
 #include "php_phongo.h"
 #include "php_bson.h"
 
-zend_class_entry *php_phongo_cursor_ce;
+zend_class_entry* php_phongo_cursor_ce;
 
 /* Check if the cursor is exhausted (i.e. ID is zero) and free any reference to
  * the session. Calling this function during iteration will allow an implicit
  * session to return to the pool immediately after a getMore indicates that the
  * server has no more results to return. */
-static void php_phongo_cursor_free_session_if_exhausted(php_phongo_cursor_t *cursor) /* {{{ */
+static void php_phongo_cursor_free_session_if_exhausted(php_phongo_cursor_t* cursor) /* {{{ */
 {
 	if (mongoc_cursor_get_id(cursor->cursor)) {
 		return;
@@ -44,7 +44,7 @@ static void php_phongo_cursor_free_session_if_exhausted(php_phongo_cursor_t *cur
 	}
 } /* }}} */
 
-static void php_phongo_cursor_free_current(php_phongo_cursor_t *cursor) /* {{{ */
+static void php_phongo_cursor_free_current(php_phongo_cursor_t* cursor) /* {{{ */
 {
 	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
 		zval_ptr_dtor(&cursor->visitor_data.zchild);
@@ -53,27 +53,18 @@ static void php_phongo_cursor_free_current(php_phongo_cursor_t *cursor) /* {{{ *
 } /* }}} */
 
 /* {{{ MongoDB\Driver\Cursor iterator handlers */
-static void php_phongo_cursor_iterator_dtor(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static void php_phongo_cursor_iterator_dtor(zend_object_iterator* iter) /* {{{ */
 {
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
+	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
 
 	if (!Z_ISUNDEF(cursor_it->intern.data)) {
-#if PHP_VERSION_ID >= 70000
 		zval_ptr_dtor(&cursor_it->intern.data);
-#else
-		zval_ptr_dtor((zval**)&cursor_it->intern.data);
-		cursor_it->intern.data = NULL;
-#endif
 	}
-
-#if PHP_VERSION_ID < 70000
-	efree(cursor_it);
-#endif
 } /* }}} */
 
-static int php_phongo_cursor_iterator_valid(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static int php_phongo_cursor_iterator_valid(zend_object_iterator* iter) /* {{{ */
 {
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
 
 	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
 		return SUCCESS;
@@ -82,34 +73,25 @@ static int php_phongo_cursor_iterator_valid(zend_object_iterator *iter TSRMLS_DC
 	return FAILURE;
 } /* }}} */
 
-static void php_phongo_cursor_iterator_get_current_key(zend_object_iterator *iter, zval *key TSRMLS_DC) /* {{{ */
+static void php_phongo_cursor_iterator_get_current_key(zend_object_iterator* iter, zval* key) /* {{{ */
 {
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
 
 	ZVAL_LONG(key, cursor->current);
 } /* }}} */
 
-#if PHP_VERSION_ID < 70000
-static void php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter, zval ***data TSRMLS_DC) /* {{{ */
+static zval* php_phongo_cursor_iterator_get_current_data(zend_object_iterator* iter) /* {{{ */
 {
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	*data = &cursor->visitor_data.zchild;
-} /* }}} */
-#else
-static zval* php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+	php_phongo_cursor_t* cursor = ((php_phongo_cursor_iterator*) iter)->cursor;
 
 	return &cursor->visitor_data.zchild;
 } /* }}} */
-#endif
 
-static void php_phongo_cursor_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static void php_phongo_cursor_iterator_move_forward(zend_object_iterator* iter) /* {{{ */
 {
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
-	php_phongo_cursor_t        *cursor = cursor_it->cursor;
-	const bson_t               *doc;
+	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
+	php_phongo_cursor_t*        cursor    = cursor_it->cursor;
+	const bson_t*               doc;
 
 	php_phongo_cursor_free_current(cursor);
 
@@ -123,38 +105,43 @@ static void php_phongo_cursor_iterator_move_forward(zend_object_iterator *iter T
 	}
 
 	if (mongoc_cursor_next(cursor->cursor, &doc)) {
-		php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the cursor is exhausted. */
+			php_phongo_cursor_free_current(cursor);
+		}
 	} else {
-		bson_error_t error;
+		bson_error_t  error = { 0 };
+		const bson_t* doc   = NULL;
 
-		if (mongoc_cursor_error(cursor->cursor, &error)) {
+		if (mongoc_cursor_error_document(cursor->cursor, &error, &doc)) {
 			/* Intentionally not destroying the cursor as it will happen
 			 * naturally now that there are no more results */
-			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
 		}
 	}
 
 	php_phongo_cursor_free_session_if_exhausted(cursor);
 } /* }}} */
 
-static void php_phongo_cursor_iterator_rewind(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+static void php_phongo_cursor_iterator_rewind(zend_object_iterator* iter) /* {{{ */
 {
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
-	php_phongo_cursor_t        *cursor = cursor_it->cursor;
-	const bson_t               *doc;
+	php_phongo_cursor_iterator* cursor_it = (php_phongo_cursor_iterator*) iter;
+	php_phongo_cursor_t*        cursor    = cursor_it->cursor;
+	const bson_t*               doc;
 
 	/* If the cursor was never advanced (e.g. command cursor), do so now */
 	if (!cursor->advanced) {
 		cursor->advanced = true;
 
-		if (!phongo_cursor_advance_and_check_for_error(cursor->cursor TSRMLS_CC)) {
+		if (!phongo_cursor_advance_and_check_for_error(cursor->cursor)) {
 			/* Exception should already have been thrown */
 			return;
 		}
 	}
 
 	if (cursor->current > 0) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot rewind after starting iteration");
+		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot rewind after starting iteration");
 		return;
 	}
 
@@ -163,7 +150,11 @@ static void php_phongo_cursor_iterator_rewind(zend_object_iterator *iter TSRMLS_
 	doc = mongoc_cursor_current(cursor->cursor);
 
 	if (doc) {
-		php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data)) {
+			/* Free invalid result, but don't return as we want to free the
+			 * session if the cursor is exhausted. */
+			php_phongo_cursor_free_current(cursor);
+		}
 	}
 
 	php_phongo_cursor_free_session_if_exhausted(cursor);
@@ -179,35 +170,28 @@ static zend_object_iterator_funcs php_phongo_cursor_iterator_funcs = {
 	NULL /* invalidate_current is not used */
 };
 
-static zend_object_iterator *php_phongo_cursor_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC) /* {{{ */
+static zend_object_iterator* php_phongo_cursor_get_iterator(zend_class_entry* ce, zval* object, int by_ref) /* {{{ */
 {
-	php_phongo_cursor_iterator *cursor_it = NULL;
-	php_phongo_cursor_t *cursor = Z_CURSOR_OBJ_P(object);
+	php_phongo_cursor_iterator* cursor_it = NULL;
+	php_phongo_cursor_t*        cursor    = Z_CURSOR_OBJ_P(object);
 
 	if (by_ref) {
 		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
 	}
 
 	if (cursor->got_iterator) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot yield multiple iterators");
+		phongo_throw_exception(PHONGO_ERROR_LOGIC, "Cursors cannot yield multiple iterators");
 		return NULL;
 	}
 
-	cursor->got_iterator = 1;
+	cursor->got_iterator = true;
 
 	cursor_it = ecalloc(1, sizeof(php_phongo_cursor_iterator));
-#if PHP_VERSION_ID >= 70000
 	zend_iterator_init(&cursor_it->intern);
-#endif
 
-#if PHP_VERSION_ID >= 70000
 	ZVAL_COPY(&cursor_it->intern.data, object);
-#else
-	Z_ADDREF_P(object);
-	cursor_it->intern.data  = (void*)object;
-#endif
 	cursor_it->intern.funcs = &php_phongo_cursor_iterator_funcs;
-	cursor_it->cursor = cursor;
+	cursor_it->cursor       = cursor;
 	/* cursor_it->current should already be allocated to zero */
 
 	php_phongo_cursor_free_current(cursor_it->cursor);
@@ -220,20 +204,20 @@ static zend_object_iterator *php_phongo_cursor_get_iterator(zend_class_entry *ce
    Sets a type map to use for BSON unserialization */
 static PHP_METHOD(Cursor, setTypeMap)
 {
-	php_phongo_cursor_t *intern;
-	php_phongo_bson_state     state = PHONGO_BSON_STATE_INITIALIZER;
-	zval                     *typemap = NULL;
-	bool                      restore_current_element = false;
-	SUPPRESS_UNUSED_WARNING(return_value) SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
+	php_phongo_cursor_t*  intern;
+	php_phongo_bson_state state;
+	zval*                 typemap                 = NULL;
+	bool                  restore_current_element = false;
 
+	PHONGO_BSON_INIT_STATE(state);
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a!", &typemap) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a!", &typemap) == FAILURE) {
 		return;
 	}
 
-	if (!php_phongo_bson_typemap_to_state(typemap, &state.map TSRMLS_CC)) {
+	if (!php_phongo_bson_typemap_to_state(typemap, &state.map)) {
 		return;
 	}
 
@@ -244,24 +228,27 @@ static PHP_METHOD(Cursor, setTypeMap)
 		restore_current_element = true;
 	}
 
+	php_phongo_bson_typemap_dtor(&intern->visitor_data.map);
+
 	intern->visitor_data = state;
 
 	/* If the cursor has a current element, we just freed it and should restore
 	 * it with a new type map applied. */
 	if (restore_current_element && mongoc_cursor_current(intern->cursor)) {
-		const bson_t *doc = mongoc_cursor_current(intern->cursor);
+		const bson_t* doc = mongoc_cursor_current(intern->cursor);
 
-		php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data);
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data)) {
+			php_phongo_cursor_free_current(intern);
+		}
 	}
 } /* }}} */
 
-static int php_phongo_cursor_to_array_apply(zend_object_iterator *iter, void *puser TSRMLS_DC) /* {{{ */
+static int php_phongo_cursor_to_array_apply(zend_object_iterator* iter, void* puser) /* {{{ */
 {
-#if PHP_VERSION_ID >= 70000
-	zval *data;
-	zval *return_value = (zval*)puser;
+	zval* data;
+	zval* return_value = (zval*) puser;
 
-	data = iter->funcs->get_current_data(iter TSRMLS_CC);
+	data = iter->funcs->get_current_data(iter);
 
 	if (EG(exception)) {
 		return ZEND_HASH_APPLY_STOP;
@@ -271,38 +258,31 @@ static int php_phongo_cursor_to_array_apply(zend_object_iterator *iter, void *pu
 	}
 	Z_TRY_ADDREF_P(data);
 	add_next_index_zval(return_value, data);
-#else
-	zval **data;
-	zval *return_value = (zval*)puser;
-
-	iter->funcs->get_current_data(iter, &data TSRMLS_CC);
-
-	if (EG(exception)) {
-		return ZEND_HASH_APPLY_STOP;
-	}
-	if (data == NULL || *data == NULL) {
-		return ZEND_HASH_APPLY_STOP;
-	}
-	Z_ADDREF_PP(data);
-	add_next_index_zval(return_value, *data);
-#endif
 
 	return ZEND_HASH_APPLY_KEEP;
 } /* }}} */
 
+static void php_phongo_cursor_id_new_from_id(zval* object, int64_t cursorid) /* {{{ */
+{
+	php_phongo_cursorid_t* intern;
+
+	object_init_ex(object, php_phongo_cursorid_ce);
+
+	intern     = Z_CURSORID_OBJ_P(object);
+	intern->id = cursorid;
+} /* }}} */
+
 /* {{{ proto array MongoDB\Driver\Cursor::toArray()
    Returns an array of all result documents for this cursor */
 static PHP_METHOD(Cursor, toArray)
 {
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
-
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
 
 	array_init(return_value);
 
-	if (spl_iterator_apply(getThis(), php_phongo_cursor_to_array_apply, (void*)return_value TSRMLS_CC) != SUCCESS) {
+	if (spl_iterator_apply(getThis(), php_phongo_cursor_to_array_apply, (void*) return_value) != SUCCESS) {
 		zval_dtor(return_value);
 		RETURN_NULL();
 	}
@@ -312,9 +292,7 @@ static PHP_METHOD(Cursor, toArray)
    Returns the CursorId for this cursor */
 static PHP_METHOD(Cursor, getId)
 {
-	php_phongo_cursor_t      *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
-
+	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -322,16 +300,14 @@ static PHP_METHOD(Cursor, getId)
 		return;
 	}
 
-	php_phongo_cursor_id_new_from_id(return_value, mongoc_cursor_get_id(intern->cursor) TSRMLS_CC);
+	php_phongo_cursor_id_new_from_id(return_value, mongoc_cursor_get_id(intern->cursor));
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Server MongoDB\Driver\Cursor::getServer()
    Returns the Server object to which this cursor is attached */
 static PHP_METHOD(Cursor, getServer)
 {
-	php_phongo_cursor_t *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
-
+	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -339,16 +315,14 @@ static PHP_METHOD(Cursor, getServer)
 		return;
 	}
 
-	phongo_server_init(return_value, intern->client, intern->server_id TSRMLS_CC);
+	phongo_server_init(return_value, intern->client, intern->server_id);
 } /* }}} */
 
 /* {{{ proto boolean MongoDB\Driver\Cursor::isDead()
    Checks if a cursor is still alive */
 static PHP_METHOD(Cursor, isDead)
 {
-	php_phongo_cursor_t      *intern;
-	SUPPRESS_UNUSED_WARNING(return_value_ptr) SUPPRESS_UNUSED_WARNING(return_value_used)
-
+	php_phongo_cursor_t* intern;
 
 	intern = Z_CURSOR_OBJ_P(getThis());
 
@@ -356,7 +330,7 @@ static PHP_METHOD(Cursor, isDead)
 		return;
 	}
 
-	RETURN_BOOL(!mongoc_cursor_is_alive(intern->cursor));
+	RETURN_BOOL(!mongoc_cursor_more(intern->cursor));
 } /* }}} */
 
 /* {{{ MongoDB\Driver\Cursor function entries */
@@ -368,25 +342,32 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursor_me[] = {
-	PHP_ME(Cursor, setTypeMap, ai_Cursor_setTypeMap, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, toArray, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Cursor_void, ZEND_ACC_PRIVATE|ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Cursor_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	/* clang-format off */
+	PHP_ME(Cursor, setTypeMap, ai_Cursor_setTypeMap, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Cursor, toArray, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Cursor_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
+	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
+	/* clang-format on */
 };
 /* }}} */
 
 /* {{{ MongoDB\Driver\Cursor object handlers */
 static zend_object_handlers php_phongo_handler_cursor;
 
-static void php_phongo_cursor_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
+static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 {
-	php_phongo_cursor_t *intern = Z_OBJ_CURSOR(object);
+	php_phongo_cursor_t* intern = Z_OBJ_CURSOR(object);
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_object_std_dtor(&intern->std);
+
+	/* If this Cursor was created in a different process, reset the client so
+	 * that mongoc_cursor_destroy does not issue a killCursors command for an
+	 * active cursor owned by a parent process. */
+	PHONGO_RESET_CLIENT_IF_PID_DIFFERS(intern);
 
 	if (intern->cursor) {
 		mongoc_cursor_destroy(intern->cursor);
@@ -416,49 +397,34 @@ static void php_phongo_cursor_free_object(phongo_free_object_arg *object TSRMLS_
 		zval_ptr_dtor(&intern->session);
 	}
 
-	php_phongo_cursor_free_current(intern);
+	php_phongo_bson_typemap_dtor(&intern->visitor_data.map);
 
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
+	php_phongo_cursor_free_current(intern);
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_cursor_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_cursor_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_cursor_t *intern = NULL;
+	php_phongo_cursor_t* intern = NULL;
 
 	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_cursor_t, class_type);
 
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
+	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
+	PHONGO_SET_CREATED_BY_PID(intern);
+
 	intern->std.handlers = &php_phongo_handler_cursor;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_cursor_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_cursor;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 
-static HashTable *php_phongo_cursor_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_cursor_get_debug_info(zval* object, int* is_temp) /* {{{ */
 {
-	php_phongo_cursor_t    *intern;
-#if PHP_VERSION_ID >= 70000
-	zval                    retval;
-#else
-	zval                    retval = zval_used_for_init;
-#endif
-
+	php_phongo_cursor_t* intern;
+	zval                 retval = ZVAL_STATIC_INIT;
 
 	*is_temp = 1;
-	intern = Z_CURSOR_OBJ_P(object);
+	intern   = Z_CURSOR_OBJ_P(object);
 
 	array_init_size(&retval, 10);
 
@@ -475,82 +441,49 @@ static HashTable *php_phongo_cursor_get_debug_info(zval *object, int *is_temp TS
 	}
 
 	if (!Z_ISUNDEF(intern->query)) {
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "query", &intern->query);
 		Z_ADDREF(intern->query);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "query", intern->query);
-		Z_ADDREF_P(intern->query);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "query");
 	}
 
 	if (!Z_ISUNDEF(intern->command)) {
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "command", &intern->command);
 		Z_ADDREF(intern->command);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "command", intern->command);
-		Z_ADDREF_P(intern->command);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "command");
 	}
 
 	if (!Z_ISUNDEF(intern->read_preference)) {
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "readPreference", &intern->read_preference);
 		Z_ADDREF(intern->read_preference);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "readPreference", intern->read_preference);
-		Z_ADDREF_P(intern->read_preference);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "readPreference");
 	}
 
 	if (!Z_ISUNDEF(intern->session)) {
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "session", &intern->session);
 		Z_ADDREF(intern->session);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "session", intern->session);
-		Z_ADDREF_P(intern->session);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "session");
 	}
 
-	ADD_ASSOC_BOOL_EX(&retval, "isDead", !mongoc_cursor_is_alive(intern->cursor));
+	ADD_ASSOC_BOOL_EX(&retval, "isDead", !mongoc_cursor_more(intern->cursor));
 
 	ADD_ASSOC_LONG_EX(&retval, "currentIndex", intern->current);
 
 	if (!Z_ISUNDEF(intern->visitor_data.zchild)) {
-#if PHP_VERSION_ID >= 70000
 		ADD_ASSOC_ZVAL_EX(&retval, "currentDocument", &intern->visitor_data.zchild);
 		Z_ADDREF(intern->visitor_data.zchild);
-#else
-		ADD_ASSOC_ZVAL_EX(&retval, "currentDocument", intern->visitor_data.zchild);
-		Z_ADDREF_P(intern->visitor_data.zchild);
-#endif
 	} else {
 		ADD_ASSOC_NULL_EX(&retval, "currentDocument");
 	}
 
 	{
-#if PHP_VERSION_ID >= 70000
 		zval server;
 
-		phongo_server_init(&server, intern->client, intern->server_id TSRMLS_CC);
+		phongo_server_init(&server, intern->client, intern->server_id);
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
-#else
-		zval *server = NULL;
-
-		MAKE_STD_ZVAL(server);
-		phongo_server_init(server, intern->client, intern->server_id TSRMLS_CC);
-		ADD_ASSOC_ZVAL_EX(&retval, "server", server);
-#endif
 	}
 
 	return Z_ARRVAL(retval);
@@ -563,20 +496,18 @@ void php_phongo_cursor_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\Driver", "Cursor", php_phongo_cursor_me);
-	php_phongo_cursor_ce = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_cursor_ce                = zend_register_internal_class(&ce);
 	php_phongo_cursor_ce->create_object = php_phongo_cursor_create_object;
 	PHONGO_CE_FINAL(php_phongo_cursor_ce);
 	PHONGO_CE_DISABLE_SERIALIZATION(php_phongo_cursor_ce);
 	php_phongo_cursor_ce->get_iterator = php_phongo_cursor_get_iterator;
 
+	zend_class_implements(php_phongo_cursor_ce, 1, php_phongo_cursor_interface_ce);
+
 	memcpy(&php_phongo_handler_cursor, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_cursor.get_debug_info = php_phongo_cursor_get_debug_info;
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_cursor.free_obj = php_phongo_cursor_free_object;
-	php_phongo_handler_cursor.offset = XtOffsetOf(php_phongo_cursor_t, std);
-#endif
-
-	zend_class_implements(php_phongo_cursor_ce TSRMLS_CC, 1, zend_ce_traversable);
+	php_phongo_handler_cursor.free_obj       = php_phongo_cursor_free_object;
+	php_phongo_handler_cursor.offset         = XtOffsetOf(php_phongo_cursor_t, std);
 } /* }}} */
 
 /*
